/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.filenametab;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.renamer.FileRenamer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * Display rename file options.
 */
public class FileOptionsPanel extends JPanel implements ActionListener {
    private static final long       serialVersionUID = 1;
    private JCheckBox               aUseTitle;
    private JCheckBox               aUseText;
    private JTextField              aUseTextInput;
    private JCheckBox               aRemoveIllegal;
    private JCheckBox               aRemoveLeading;
    private JCheckBox               aRemoveTrailing;
    private JCheckBox               aRemoveText;
    private JTextField              aRemoveTextInput;
    private JCheckBox               aReplaceText;
    private JTextField              aReplaceTextInput;
    private JTextField              aInsertAlbumText;
    private JCheckBox               aInsertAlbum;
    private JCheckBox               aInsertArtist;
    private JTextField              aInsertArtistText;
    private JCheckBox               aInsertText;
    private JTextField              aInsertTextInput;
    private JCheckBox               aAppendText;
    private JTextField              aAppendTextInput;
    private JRadioButton            aNumber0;
    private JRadioButton            aNumber1;
    private JRadioButton            aNumber2;
    private JRadioButton            aNumber3;
    private JRadioButton            aNumber4;
    private JTextField              aStartNumber2;
    private JTextField              aStartNumber4;
    private JTextField              aNumberSizeText;
    private JTextField              aNumberSeparator;
    private JComboBox               aExtension;
    private JComboBox               aCapitalize;


    /**
     * Create rename options panel
     */
    public FileOptionsPanel() {
        super();
        JPanel top = new JPanel();
        JPanel bottom = new JPanel();

        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
        bottom.setLayout(new BoxLayout(bottom, BoxLayout.Y_AXIS));

        /*********************************************************************/

        aUseTitle = ComponentFactory.createCheck(false, "Use Title", "If title exist, use it as the filename");
        top.add(ComponentFactory.createOnePanel(aUseTitle));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aUseText = ComponentFactory.createCheck(false, "Set Filename", "Enter text to use as the filename");
        aUseTextInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aUseText, aUseTextInput));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aRemoveIllegal = ComponentFactory.createCheck(true, "Remove Illegal Characters", "Remove illegal characters such as : and / and a few more", 0, 0);
        top.add(ComponentFactory.createOnePanel(aRemoveIllegal));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aRemoveLeading = ComponentFactory.createCheck(false, "Remove Leading", "Remove all leading none letters from the filename until first letter is found");
        top.add(ComponentFactory.createOnePanel(aRemoveLeading));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aRemoveTrailing = ComponentFactory.createCheck(false, "Remove Trailing", "Remove all trailing none letters from the filename");
        top.add(ComponentFactory.createOnePanel(aRemoveTrailing));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aRemoveText = ComponentFactory.createCheck(false, "Remove Text", "<html>Enter text to remove from the filename<br>Try regular expressions such ' - \\d* of \\d*' which removes text like ' - 09 of 12'</html>");
        aRemoveTextInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aRemoveText, aRemoveTextInput));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aReplaceText = ComponentFactory.createCheck(false, "Replace Text", "Enter text to replace removed text");
        aReplaceTextInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aReplaceText, aReplaceTextInput));

        /*********************************************************************/

        top.add(Box.createRigidArea(new Dimension(0, 10)));
        top.add(new JSeparator());
        top.add(Box.createRigidArea(new Dimension(0, 10)));

        /*********************************************************************/

        aInsertAlbum = ComponentFactory.createCheck(false, "Insert Album", "<html>Insert album before filename<br>Enter optional separator between album and filename<br>Default separator is ' - '</html>");
        aInsertAlbumText = ComponentFactory.createInput(" - ", "");
        top.add(ComponentFactory.createTwoPanel(aInsertAlbum, aInsertAlbumText));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aInsertArtist = ComponentFactory.createCheck(false, "Insert Artist", "<html>Insert artist before filename<br>Enter optional separator between artist and filename<br>Default separator is ' - '</html>");
        aInsertArtistText = ComponentFactory.createInput(" - ", "");
        top.add(ComponentFactory.createTwoPanel(aInsertArtist, aInsertArtistText));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aInsertText = ComponentFactory.createCheck(false, "Insert Text", "Enter text to insert before filename");
        aInsertTextInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aInsertText, aInsertTextInput));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aAppendText = ComponentFactory.createCheck(false, "Append Text", "Enter text to append after filename");
        aAppendTextInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aAppendText, aAppendTextInput));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        top.add(Box.createRigidArea(new Dimension(0, 10)));
        top.add(new JSeparator());
        top.add(Box.createRigidArea(new Dimension(0, 10)));

        /*********************************************************************/

        aNumber0 = ComponentFactory.createRadio(true, "Don't Insert Track Number", "");
        top.add(ComponentFactory.createOnePanel(aNumber0));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aNumber1 = ComponentFactory.createRadio(false, "Insert Track Number", "<html>Insert track number from the track first in the filename</html>");
        top.add(ComponentFactory.createOnePanel(aNumber1));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aNumber2 = ComponentFactory.createRadio(false, "Insert Number", "<html>Insert track number first in the filename<br>Enter start track number<br>Default start track is 1</html>");
        aStartNumber2 = ComponentFactory.createInput("1", "");
        top.add(ComponentFactory.createTwoPanel(aNumber2, aStartNumber2));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aNumber3 = ComponentFactory.createRadio(false, "Append Track Number", "<html>Append track number from the track last in the filename</html>");
        top.add(ComponentFactory.createOnePanel(aNumber3));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aNumber4 = ComponentFactory.createRadio(false, "Append Number", "<html>Append track number last in the filename<br>Enter start track number<br>Default start track is 1</html>");
        aStartNumber4 = ComponentFactory.createInput("1", "");
        top.add(ComponentFactory.createTwoPanel(aNumber4, aStartNumber4));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        ButtonGroup group = new ButtonGroup();
        group.add(aNumber0);
        group.add(aNumber1);
        group.add(aNumber2);
        group.add(aNumber3);
        group.add(aNumber4);

        JLabel label = ComponentFactory.createLabel("Number Size", "<html>Number of numerals in the track number<br>Enter a number between 0 and 6</html>");
        aNumberSizeText = ComponentFactory.createInput("2", "");
        top.add(ComponentFactory.createTwoPanel(label, aNumberSizeText));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        label = ComponentFactory.createLabel("Number Separator", "<html>Separator string to insert between track number and filename<br>Default separator is space</html>");
        aNumberSeparator = ComponentFactory.createInput(" ", "");
        top.add(ComponentFactory.createTwoPanel(label, aNumberSeparator));

        /*********************************************************************/

        top.add(Box.createRigidArea(new Dimension(0, 10)));
        top.add(new JSeparator());
        top.add(Box.createRigidArea(new Dimension(0, 10)));

        /*********************************************************************/

        aCapitalize = ComponentFactory.createCombo(Constants.OPTIONS_CAP_FILE, 0, "Change filename capitalisation", 0, 0);
        top.add(ComponentFactory.createOnePanel(aCapitalize));
        top.add(Box.createRigidArea(new Dimension(0, 3)));

        aExtension = ComponentFactory.createCombo(Constants.OPTIONS_CAP_EXT, 0, "Change file extension capitalisation", 0, 0);
        top.add(ComponentFactory.createOnePanel(aExtension));

        /*********************************************************************/

        JButton button = ComponentFactory.createButton("Reset &Options", "Reset and clear all options", this, 0, 0);
        bottom.add(button);

        /*********************************************************************/

        add(top, BorderLayout.NORTH);
        add(bottom, BorderLayout.SOUTH);

        setMinimumSize(new Dimension(150, Short.MAX_VALUE));
        setPreferredSize(new Dimension(250, Short.MAX_VALUE));
    }


    /**
     * Reset options panel.
     *
     * @param actionEvent
     */
    @Override public void actionPerformed(ActionEvent actionEvent) {
        aUseTitle.setSelected(false);
        aUseText.setSelected(false);
        aInsertAlbum.setSelected(false);
        aInsertArtist.setSelected(false);
        aRemoveIllegal.setSelected(true);
        aRemoveLeading.setSelected(false);
        aRemoveTrailing.setSelected(false);
        aRemoveText.setSelected(false);
        aReplaceText.setSelected(false);
        aInsertText.setSelected(false);
        aAppendText.setSelected(false);
        aNumber0.setSelected(true);

        aUseTextInput.setText("");
        aInsertAlbumText.setText(" - ");
        aInsertArtistText.setText(" - ");
        aRemoveTextInput.setText("");
        aReplaceTextInput.setText("");
        aInsertTextInput.setText("");
        aAppendTextInput.setText("");
        aStartNumber2.setText("1");
        aStartNumber4.setText("1");
        aNumberSizeText.setText("2");
        aNumberSeparator.setText(" ");

        aExtension.setSelectedIndex(0);
        aCapitalize.setSelectedIndex(0);
    }


    /**
     * Create renamer object with all options set.
     *
     * @return FileRenamer object.
     * @see FileRenamer
     */
    public FileRenamer getRenamer() {
        FileRenamer renamer = new FileRenamer();

        if (aUseTitle.isSelected()) {
            renamer.aUseTitle = "YES";
        }

        if (aUseText.isSelected()) {
            renamer.aUseText = aUseTextInput.getText();
        }

        if (aRemoveIllegal.isSelected()) {
            renamer.aRemoveIllegal = "YES";
        }

        if (aRemoveLeading.isSelected()) {
            renamer.aRemoveLeading = "YES";
        }

        if (aRemoveTrailing.isSelected()) {
            renamer.aRemoveTrailing = "YES";
        }

        if (aRemoveText.isSelected()) {
            renamer.aRemoveText = aRemoveTextInput.getText();
        }

        if (aReplaceText.isSelected()) {
            renamer.aReplaceText = aReplaceTextInput.getText();
        }

        if (aInsertAlbum.isSelected()) {
            renamer.aUseAlbum = aInsertAlbumText.getText();
        }

        if (aInsertArtist.isSelected()) {
            renamer.aUseArtist = aInsertArtistText.getText();
        }

        if (aInsertText.isSelected()) {
            renamer.aInsertText = aInsertTextInput.getText();
        }

        if (aAppendText.isSelected()) {
            renamer.aAppendText = aAppendTextInput.getText();
        }

        if (aNumber1.isSelected()) {
            renamer.aNumber = 1;
        }

        else if (aNumber2.isSelected()) {
            renamer.aNumber = 2;
            renamer.aStartNumber = aStartNumber2.getText();
        }
        else if (aNumber3.isSelected()) {
            renamer.aNumber = 3;
        }
        else if (aNumber4.isSelected()) {
            renamer.aNumber = 4;
            renamer.aStartNumber = aStartNumber4.getText();
        }

        renamer.aNumberSize = aNumberSizeText.getText();
        renamer.aNumberSeparator = aNumberSeparator.getText();
        renamer.aExtension = aExtension.getSelectedIndex();
        renamer.aCapitalize = aCapitalize.getSelectedIndex();

        return renamer;
    }
}
