/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.leftpanel;

import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.gui.dlg.ProgressDialog;
import com.googlepages.dronten.jtagger.thread.AudioThread;
import com.googlepages.dronten.jtagger.thread.BaseThread;
import com.googlepages.dronten.jtagger.thread.DirectoryReaderThread;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Progress;

import javax.swing.*;
import java.awt.*;


/**
 * Container for DirTree.
 *
 * @see com.googlepages.dronten.jtagger.gui.leftpanel.DirTree
 */
public class LeftPanel extends JPanel implements DirListener {
    private static final long   serialVersionUID = 1;
    private String aCurrentDirectory = null;
    private DirTree             aDirTree = null;

    /**
     * Create panel and set minimum width size.
     * Left DirTree load root file system.
     */
    public LeftPanel() {
        super();
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 3, 5, 3));
        aDirTree = new DirTree("My Computer");
        add(aDirTree, BorderLayout.CENTER);

        aDirTree.addDirListener(this);
        aDirTree.initRoot();

        setMinimumSize(new Dimension(150, Short.MAX_VALUE));
        setPreferredSize(new Dimension(300, Short.MAX_VALUE));
    }


    /**
     *
     * @return Get DirTree object
     */
    public DirTree getDirTree() {
        return aDirTree;
    }


    /**
     *
     * @return Get DirTree object
     */
    public String getCurrentDirectory() {
        return aCurrentDirectory;
    }


    /**
     * Reload current directory.
     */
    public void reload() {
        valueChanged(aCurrentDirectory);
    }


    /**
     *
     * @param currentDirectory Set current directory
     */
    public void setCurrentDirectory(String currentDirectory) {
        aCurrentDirectory = currentDirectory;
    }


    /**
     * New directory has been selected.
     * Member String aCurrDir must be set no a none null value to allow actual loading of files.
     *
     * @param path
     */
    @Override public void valueChanged(String path) {
        AudioThread.get().stopPlaying();

        if (path != null) {
            aCurrentDirectory = path;
            Progress.get().clear();
            ProgressDialog dlg = new ProgressDialog(JTagger.get(), "Reading Directory", "Cancel", true);
            BaseThread work = new DirectoryReaderThread(Log.get(), Progress.get(), aCurrentDirectory);

            dlg.centerOnApplication();
            dlg.show(work);

            try {
                work.join();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
