/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.model;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.audio.AudioTrackComparator;
import com.googlepages.dronten.jtagger.gui.dlg.ProgressDialog;
import com.googlepages.dronten.jtagger.renamer.TitleRenamer;
import com.googlepages.dronten.jtagger.thread.AudioThread;
import com.googlepages.dronten.jtagger.thread.BaseThread;
import com.googlepages.dronten.jtagger.thread.BatchWorkThread;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Progress;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;


/**
 * TitleModel uses mp3 model from the only instance of TrackModel that exist.
 */
public class TitleModel extends AbstractTableModel implements ActionListener {
    private static final long   serialVersionUID = 1;
    public MouseHandler         aMouseHandler;


    /**
     * Singleton object.
     */
    public TitleModel() {
        aMouseHandler = new MouseHandler();
    }


    /**
     * Button callback.
     *
     * @param event Action event
     */
    @Override public void actionPerformed(ActionEvent event) {
        if (event.getSource() == JTagger.get().getMainWindow().getTitlePane().getTitleTable().aTestRenameButton) {
            try {
                TitleRenamer renamer = JTagger.get().getMainWindow().getTitlePane().getTitleOptionsPanel().getRenamer();

                JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().startTest();
                for (AudioTrack track : JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTestAudioTracks()) {
                    if (track.isSelected()) {
                        renamer.rename(track);
                    }
                }

                JTagger.get().setMessage(Constants.MESSAGE_TEST_OK, false);
                JTagger.get().reloadAllTables();
            }
            catch (Exception e) {
                Log.get().addTime(1, e.getMessage());
                JTagger.get().setMessage(e.getMessage(), true);
                JTagger.get().reloadAllTables();
            }
        }
        else if (event.getSource() == JTagger.get().getMainWindow().getTitlePane().getTitleTable().aUndoTestButton) {
            JTagger.get().setMessage(Constants.MESSAGE_TEST_UNDONE, false);
            JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().clearTest();
            JTagger.get().reloadAllTables();
        }
        else if (event.getSource() == JTagger.get().getMainWindow().getTitlePane().getTitleTable().aUpdateButton) {
            try {
                AudioThread.get().throwIfPlaying();

                ProgressDialog  dlg = new ProgressDialog(JTagger.get(), "Updating Tags", "Cancel", true);
                BaseThread      work = new BatchWorkThread(Log.get(), Progress.get(), JTagger.get().getMainWindow().getTitlePane().getTitleOptionsPanel().getRenamer());

                dlg.centerOnApplication();
                dlg.show(work);
                work.join();
                JTagger.get().reloadAllTables();
            }
            catch (Exception e) {
                Log.get().addTime(1, e.getMessage());
                JTagger.get().setMessage(e.getMessage(), true);
                JTagger.get().reloadAllTables();
            }
        }
        else {
            assert false;
        }
    }


    /**
     * @param column
     * @return Column class
     */
    @Override public Class getColumnClass(int column) {
        switch (column) {
            case 0:
                return Boolean.class;

            case 1:
                return Integer.class;

            default:
                return String.class;
        }
    }


    /**
     * @return column count
     */
    @Override public int getColumnCount() {
        return 3;
    }


    /**
     * @param col
     * @return Column label
     */
    @Override public String getColumnName(int col) {
        switch (col) {
            case 0:
                return "Select";

            case 1:
                return "Track";

            case 2:
                return "Title";

            default:
                assert false;
                return "";
        }
    }


    /**
     * @return number of rows
     */
    @Override public int getRowCount() {
        return JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getRowCount();
    }


    /**
     * @param row
     * @param col
     * @return get object to paint
     */
    @Override public Object getValueAt(int row, int col) {
        AudioTrack track = JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTrack(row);
        AudioTrack track_ = JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTestTrack(row);

        try {
            if (track != null) {
                if (col == 0) {
                    return track.isSelected();
                }
                else if (col == 1) {
                    if (track_ != null) {
                        return track_.aTrack;
                    }
                    else {
                        return track.aTrack;
                    }
                }
                else if (col == 2) {
                    if (track_ != null) {
                        return track_.aTitle;
                    }
                    else {
                        return track.aTitle;
                    }
                }
            }
        }
        catch (Exception e) {
            if (Constants.DEBUG) e.printStackTrace();
        }

        return "";
    }


    /**
     * Edit only column 0, the select column.
     *
     * @param row
     * @param col
     * @return true if cell is editable
     */
    @Override public boolean isCellEditable(int row, int col) {
        switch (col) {
            case 0:
            case 1:
            case 2:
                return true;

            default:
                return false;
        }
    }


    /**
     * @param value
     * @param row
     * @param col
     */
    @Override public void setValueAt(Object value, int row, int col) {
        AudioTrack  track = JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getTrack(row);
        boolean     write = true;

        try {
            if (track != null) {
                if (col == 0) {
                    write = false;
                    track.setSelected((Boolean) value);
                }
                else if (col == 1) {
                    Log.get().addTime(2, String.format("Changing track for file '%s' from '%s' to '%s'", track.aFileName, track.aTrack, value.toString()));
                    track.aTrack = value.toString();
                }
                else if (col == 2) {
                    Log.get().addTime(2, String.format("Changing title for file '%s' from '%s' to '%s'", track.aFileName, track.aTitle, (String) value));
                    track.aTitle = (String) value;
                }

                if (write) {
                    track.write(track.aType);
                    JTagger.get().setMessage(Constants.MESSAGE_FILERENAME_OK, false);
                    JTagger.get().reloadAllTables();
                }
            }
        }
        catch (Exception e) {
            Log.get().addTime(1, e.getMessage());
            JTagger.get().setMessage(e.getMessage(), true);
            JTagger.get().reloadAllTables();
        }
    }


    /**
     * Handle mouse clicks on the table header.<br>
     * Do a sort on the column type.<br>
     */
    private class MouseHandler extends MouseAdapter {
        /**
         *
         * @param mouseEvent
         */
        @Override public void mouseClicked(MouseEvent mouseEvent) {
            JTableHeader        header = (JTableHeader) mouseEvent.getSource();
            TableColumnModel    columnModel = header.getColumnModel();
            int                 viewColumn = columnModel.getColumnIndexAtX(mouseEvent.getX());
            int                 column = columnModel.getColumn(viewColumn).getModelIndex();

            if (column != -1) {
                switch (column) {
                    case 0:
                        JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getComparator().aSortType = AudioTrackComparator.SORT.COMPARE_SELECTED;
                        break;

                    case 1:
                        JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getComparator().aSortType = AudioTrackComparator.SORT.COMPARE_TRACK;
                        break;

                    case 2:
                        JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getComparator().aSortType = AudioTrackComparator.SORT.COMPARE_TITLE;
                        break;
                }

                if (mouseEvent.isControlDown()) {
                    JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().setSortAscending(false);
                    JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().sort();
                }
                else {
                    JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().setSortAscending(true);
                    JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().sort();
                }
            }
        }
    }
}
