/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.thread;

import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.util.Log;
import javazoom.jl.player.Player;

import java.io.FileInputStream;


/**
 * Audioplaying thread.
 */
public class AudioThread extends BaseThread {
    static private AudioThread  AUDIO_THREAD = null;
    private Player              aPlayer = null;
    private AudioTrack          aTrack = null;
    private final Boolean       aSync = false;
    private String              aExceptionMessage = "";
    public boolean              aPlaying = false;


    /**
     * Create audio thread.
     *
     * @param message
     */
    private AudioThread(String message) {
        super(null, null);
        aExceptionMessage = message;
    }


    /**
     * Get one and only instance of AudioThread
     *
     * @return Singleton instance
     */
    public static AudioThread get() {
        return get("");
    }


    /**
     * Get one and only instance of AudioThread
     *
     * @param message
     * @return Singleton instance
     */
    public static AudioThread get(String message) {
        if (AUDIO_THREAD == null) {
            AUDIO_THREAD = new AudioThread(message);
            AUDIO_THREAD.start();
        }
        return AUDIO_THREAD;
    }


    /**
     * @throws Exception
     */
    public void throwIfPlaying() throws Exception {
        if (aPlayer != null && !aPlayer.isComplete()) {
            throw new Exception(aExceptionMessage);
        }
    }


    /**
     *
     */
    public void run() {
        aRunning = true;
        while (aRunning) {
            try {
                Thread.sleep(50);
            }
            catch (InterruptedException e) {
            }

            synchronized (aSync) {
                if (aTrack != null) {
                    stopPlaying();

                    try {
                        aPlayer = new Player(new FileInputStream(aTrack.aFullFileName));
                        JTagger.get().setMessage("Playing " + ((aTrack.aTitle.length() > 0) ? aTrack.aTitle : aTrack.aFileName), false);
                        aTrack = null;
                    }
                    catch (Exception e) {
                        Log.get().addTime(1, e.getMessage());
                    }
                }
            }

            if (aPlayer != null) {
                try {
                    aPlayer.play();
                    aPlaying = !aPlayer.isComplete();
                }
                catch (Exception e) {
                    Log.get().addTime(1, e.getMessage());
                }

                synchronized (aSync) {
                    aPlayer = null;
                }
            }
        }
    }


    /**
     *
     */
    public void stopPlaying() {
        synchronized (aSync) {
            if (aPlayer != null) {
                aPlayer.close();
            }

            aPlaying = false;
        }
    }


    /**
     * Sstart playing new track;
     *
     * @param track
     */
    public void setTrack(AudioTrack track) {
        synchronized (aSync) {
            stopPlaying();
            aTrack = track;
        }
    }
}
